
# -*- codeing = utf-8 -*-
# @File：hhDouban.py
# @Ver：1.0.0
# @Author：立树
# @Time：2021/6/11 17:01
# @IDE：PyCharm

import re
import execjs
from .hhAjax import hhGet

class hhDouban(object):

    def __init__(self):
        pass

    def search(self,opt={}):
        # 配置
        hhOpt = {
            "search": "",
            "decrypt": ""
        }
        hhOpt.update(opt)

        # 参数判断
        if hhOpt["search"]=="" or hhOpt["decrypt"]=="":
            print("hhframe.hhDouban Error - 请补全参数（search、decrypt）")
            return {}

        try:
            # 发起请求
            page = hhGet(f"https://search.douban.com/movie/subject_search?search_text={hhOpt['search']}&cat=1002")

            # 获取加密数据
            encrypt = re.search('window.__DATA__ = "([^"]+)"',page).group(1)
            # print(encrypt)

            # 提取解密 js
            with open(hhOpt["decrypt"], "r", encoding="UTF-8", errors="ignore") as f:
                decrypt_js = f.read()
                # print(decrypt_js)

            # 解密数据
            ctx = execjs.compile(decrypt_js)
            ret = ctx.call("decrypt",encrypt)
            return ret["payload"]
        except Exception as err:
            print(f"hhframe.hhDouban Error - {err}")
            return {}




# -*- codeing = utf-8 -*-
# @File：hhMysql.py
# @Ver：1.0.0
# @Author：立树
# @Time：2021/6/10 1:32
# @IDE：PyCharm

import MySQLdb

class hhMysql(object):

    conn = None
    hhOpt = {
        "host": "",
        "port": 3306,
        "username": "",
        "password": "",
        "database": ""
    }

    def __init__(self,opt={}):
        try:
            # 配置
            self.hhOpt.update(opt)
            # 连接数据库
            self.connect()
        except Exception as err:
            self.__Error(err)

    # 连接数据库
    def connect(self):
        # 参数判断
        host,port,username,password,database = self.hhOpt.values()
        if host=="" or port=="" or username=="" or password=="" or database=="":
            self.__Error("请补全数据库参数（host、port、username、password、database）")
            return

        try:
            self.conn = MySQLdb.connect(
                host = host,
                port = port,
                user = username,
                passwd = password,
                db = database,
                charset = "utf8"
            )
        except MySQLdb.Error as err:
            self.__Error(f"Error {err.args[0]}: {err.args[1]}")

    # 关闭连接
    def close(self):
        if self.conn:
            self.conn.close()

    # 查询数据
    def query(self,sql=""):
        # 参数判断
        if sql=="":
            return self.__Response({
                "msg": "fail"
            })

        # 执行 SQL
        Command = {
            "select": "查询成功",
            "insert": "插入成功",
            "update": "修改成功",
            "delete": "删除成功"
        }
        command = sql[0:6].lower()
        cursor = self.conn.cursor()
        try:
            if command=="select":
                cursor.execute(sql)
                rows = cursor.fetchall()
                cols = [key[0] for key in cursor.description]
                return self.__Response({
                    "result": [dict(zip(cols,row)) for row in rows],
                    "nums": cursor.rowcount,
                    "rows": rows,
                    "cols": cols,
                    "sql": sql,
                    "msg": Command[command]
                })
            else:
                cursor.execute(sql)
                self.conn.commit()
                return self.__Response({
                    "nums": cursor.rowcount,
                    "sql": sql,
                    "msg": Command[command] if cursor.rowcount>0 else "ok"
                })
        except Exception as err:
            self.__Error(f"Error {err.args[0]}: {err.args[1]}")
            self.conn.rollback()
            return self.__Response({
                "msg": err
            })
        finally:
            cursor.close()

    # 执行结果
    def __Response(self,opt={}):
        hhRet = {
            "result": [],
            "nums": 0,
            "rows": tuple(),
            "cols": [],
            "sql": "",
            "msg": ""
        }
        hhRet.update(opt)
        # print(hhRet)
        return hhRet

    # 错误处理
    def __Error(self,msg):
        print("hhframe.hhMysql Error - ", msg)



# -*- codeing = utf-8 -*-
# @File：hhAjax.py
# @Ver：1.0.0
# @Author：立树
# @Time：2021/5/26 13:38
# @IDE：PyCharm

# 测试网站：http://httpbin.org

import urllib.request,urllib.error,urllib.parse

def hhGet(url,decode="utf-8"):
    # 请求数据
    html = ""
    request = urllib.request.Request(
        url = urllib.request.quote(url,safe=";/?:@&=+$,",encoding="utf-8"),
        method = "GET",
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36"
        }
    )
    # 发起请求
    try:
        response = urllib.request.urlopen(request,timeout=30)
        html = response.read().decode(decode,"ignore")
    except urllib.error.URLError as err:
        if hasattr(err,"code"):
            print(err.code)
        if hasattr(err,"reason"):
            print(err.reason)
    return html


def hhPost(url,data={},decode="utf-8"):
    # 请求数据
    html = ""
    request = urllib.request.Request(
        url,
        data=bytes(urllib.parse.urlencode(data),encoding="utf-8"),
        method="POST",
        headers={
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36"
        }
    )

    # 发起请求
    try:
        response = urllib.request.urlopen(request)
        # 对获取到的网页源码进行 utf-8 解码
        html = response.read().decode(decode,"ignore")
    except urllib.error.URLError as err:
        if hasattr(err,"code"):
            print(err.code)
        if hasattr(err,"reason"):
            print(err.reason)
    return html




# -*- codeing = utf-8 -*-
# @File：hhExcel.py
# @Ver：1.0.0
# @Author：立树
# @Time：2021/5/27 16:26
# @IDE：PyCharm

import xlwt

# 设置单元格高度
def setXlsCellHeight(row,height):
    row.height_mismatch = True
    row.height = height * 20                # 20为基准数

# 设置单元格宽度
def setXlsCellsWidth(sheet,widths):
    for i,width in enumerate(widths):
        sheet.col(i).width = width*256      # 256为基准数

# 单元格样式
def getXlsCellStyle(font={},align={}):
    # 字体默认配置
    hhFont = {
        "family": "微软雅黑",
        "size": 10,
    }
    hhAlign = {
        "horz": "left",
        "vert": "center"
    }
    hhAlignHorz = {
        "left": xlwt.Alignment.HORZ_LEFT,
        "center": xlwt.Alignment.HORZ_CENTER,
        "right": xlwt.Alignment.HORZ_RIGHT,
        "filled": xlwt.Alignment.HORZ_FILLED,
        "general": xlwt.Alignment.HORZ_GENERAL,
        "center_across_sel": xlwt.Alignment.HORZ_CENTER_ACROSS_SEL,
        "distributed": xlwt.Alignment.HORZ_DISTRIBUTED,
        "justified": xlwt.Alignment.HORZ_JUSTIFIED
    }
    hhAlignVert = {
        "top": xlwt.Alignment.VERT_TOP,
        "center": xlwt.Alignment.VERT_CENTER,
        "bottom": xlwt.Alignment.VERT_BOTTOM,
        "justified": xlwt.Alignment.VERT_JUSTIFIED,
        "distributed": xlwt.Alignment.VERT_DISTRIBUTED
    }

    hhFont.update(font)
    hhAlign.update(align)

    font = xlwt.Font()
    font.name = hhFont["family"]            # 字体名称
    font.height = hhFont["size"] * 20       # 字体大小，10为字号，20为衡量单位
    # font.colour_index = 4                 # 字体颜色

    align = xlwt.Alignment()
    align.horz = hhAlignHorz[hhAlign["horz"]]
    align.vert = hhAlignVert[hhAlign["vert"]]

    style = xlwt.XFStyle()
    style.font = font
    style.alignment = align

    return style




# -*- codeing = utf-8 -*-
# @File：hhQiniu.py
# @Ver：1.0.0
# @Author：立树
# @Time：2021/6/9 23:54
# @IDE：PyCharm

# Python SDK
# https://developer.qiniu.com/kodo/1242/python

from qiniu import Auth,BucketManager,put_file

class hhQiniu(object):

    __hhAuth = None
    __hhOpt = {
        "ak": "",
        "sk": "",
        "bucket_url": ""
    }

    def __init__(self,opt={}):
        try:
            self.__hhOpt.update(opt)
            self.__hhAuth = Auth(self.__hhOpt["ak"],self.__hhOpt["sk"])
        except Exception as err:
            print("hhframe.hhQiniu Error - ",err)

    def __Response(self,opt={}):
        hhRet = {
            "file": "",
            "url": "",
            "key": "",
            "response": {},
            "msg": ""
        }
        hhRet.update(opt)
        print("hhframe.hhQiniu Result - ",hhRet["msg"])
        return hhRet

    # 上传本地图片
    def qnUpload(self,opt={}):
        # 默认参数
        hhOpt = {
            "file": "",
            "bucket": "",
            "key": "",
            "callback": ""
        }
        hhOpt.update(opt)

        file = hhOpt["file"]
        bucket = hhOpt["bucket"]
        key = hhOpt["key"]

        # 参数判断
        if self.__hhAuth==None:
            return self.__Response({
                "msg": "上传失败，密钥无效"
            })
        if file=="" or bucket=="" or key=="":
            return self.__Response({
                "msg": "上传失败，参数异常"
            })

        # 上传
        try:
            # 生成上传 Token，可以指定过期时间等
            token = self.__hhAuth.upload_token(bucket, key, 3600)

            # 要上传文件的本地路径
            ret,response = put_file(token,key,file,version="v2")

            # print(ret)
            # print(response)

            return self.__Response({
                "file": file,
                "url": self.__hhOpt["bucket_url"]+ret["key"],
                "key": ret["key"],
                "response": response,
                "msg": "上传成功" if response.status_code == 200 else "上传失败"  # 模拟的三元运算符
            })
        except Exception as err:
            return self.__Response({
                "msg": err
            })

    # 拉取网络图片
    def qnFetch(self,opt={}):
        # 默认参数
        hhOpt = {
            "url": "",
            "bucket": "",
            "key": "",
            "callback": ""
        }
        hhOpt.update(opt)

        url = hhOpt["url"]
        bucket = hhOpt["bucket"]
        key = hhOpt["key"]

        # 参数判断
        if self.__hhAuth==None:
            return self.__Response({
                "msg": "上传失败，密钥无效"
            })
        if url=="" or bucket=="" or key=="":
            return self.__Response({
                "msg": "上传失败，参数异常"
            })

        # 抓取文件
        Bucket = BucketManager(self.__hhAuth)
        ret,response = Bucket.fetch(url,bucket,key)

        # print(file)
        # print(response)

        if response.status_code == 200:
            return self.__Response({
                "file": url,
                "url": self.__hhOpt["bucket_url"]+ret["key"],
                "key": ret["key"],
                "response": response,
                "msg": "上传成功"
            })
        else:
            return self.__Response({
                "file": url,
                "response": response,
                "msg": response.text_body
            })
